# Licensed under a 3-clause BSD style license - see LICENSE.rst
import sys
import time

from threading import Thread

from six.moves import BaseHTTPServer, SimpleHTTPServer
from six.moves.urllib.request import urlopen

import pytest

from pytest_remotedata.disable_internet import no_internet


def test_outgoing_fails():
    with pytest.raises(IOError):
        with no_internet():
            urlopen('http://www.python.org')


class StoppableHTTPServer(BaseHTTPServer.HTTPServer, object):
    def __init__(self, *args):
        super(StoppableHTTPServer, self).__init__(*args)
        self.stop = False

    def handle_request(self):
        self.stop = True
        super(StoppableHTTPServer, self).handle_request()

    def serve_forever(self):
        """
        Serve until stop set, which will happen if any request is handled
        """
        while not self.stop:
            self.handle_request()


@pytest.mark.parametrize(('localhost'), ('localhost', '127.0.0.1'))
def test_localconnect_succeeds(localhost):
    """
    Ensure that connections to localhost are allowed, since these are genuinely
    not remotedata.
    """

    # port "0" means find open port
    # see http://stackoverflow.com/questions/1365265/on-localhost-how-to-pick-a-free-port-number
    httpd = StoppableHTTPServer(('localhost', 0),
                                SimpleHTTPServer.SimpleHTTPRequestHandler)

    port = httpd.socket.getsockname()[1]

    server = Thread(target=httpd.serve_forever)
    server.setDaemon(True)

    server.start()
    time.sleep(0.1)

    urlopen('http://{localhost:s}:{port:d}'.format(localhost=localhost, port=port)).close()


PY3_4 = sys.version_info[:2] >= (3, 4)


# Used for the below test--inline functions aren't pickleable
# by multiprocessing?
def _square(x):
    return x ** 2


@pytest.mark.skipif('not PY3_4 or sys.platform == "win32" or sys.platform.startswith("gnu0")')
def test_multiprocessing_forkserver():
    """
    Test that using multiprocessing with forkserver works.  Perhaps
    a simpler more direct test would be to just open some local
    sockets and pass something through them.

    Regression test for https://github.com/astropy/astropy/pull/3713
    """

    import multiprocessing
    ctx = multiprocessing.get_context('forkserver')
    pool = ctx.Pool(1)
    result = pool.map(_square, [1, 2, 3, 4, 5])
    pool.close()
    pool.join()
    assert result == [1, 4, 9, 16, 25]



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# this test doesn't actually use any online data, it should just be skipped
# by run_tests because it has the remote_data decorator.

import pytest
from contextlib import closing
from six.moves.urllib.request import urlopen


ASTROPY_DATA_URL = "http://data.astropy.org/"
TIMEOUT = 10


def download_file(remote_url):
    with closing(urlopen(remote_url, timeout=TIMEOUT)) as remote:
        remote.read()

@pytest.mark.remote_data
def test_skip_remote_data(pytestconfig):

    # astropy.test() has remote_data=none or remote_data=astropy but we still
    # got here somehow, so fail with a helpful message

    if pytestconfig.getoption('remote_data') == 'none':
        pytest.fail('@remote_data was not skipped with remote_data=none')
    elif pytestconfig.getoption('remote_data') == 'astropy':
        pytest.fail('@remote_data was not skipped with remote_data=astropy')

    # Test Astropy URL
    download_file(ASTROPY_DATA_URL + 'galactic_center/gc_2mass_k.fits')

    # Test non-Astropy URL
    download_file('http://www.google.com')


@pytest.mark.remote_data(source='astropy')
def test_skip_remote_data_astropy(pytestconfig):

    # astropy.test() has remote_data=none but we still got here somehow,
    # so fail with a helpful message

    if pytestconfig.getoption('remote_data') == 'none':
        pytest.fail('@remote_data was not skipped with remote_data=none')

    # Test Astropy URL
    download_file(ASTROPY_DATA_URL + 'galactic_center/gc_2mass_k.fits')

    # Test non-Astropy URL
    if pytestconfig.getoption('remote_data') == 'astropy':
        with pytest.raises(Exception) as exc:
            download_file('http://www.google.com')
        assert "An attempt was made to connect to the internet" in str(exc.value)
    else:
        download_file('http://www.google.com')


@pytest.mark.internet_off
def test_internet_off_decorator(pytestconfig):
    # This test should only run when internet access has been disabled
    if pytestconfig.getoption('remote_data') != 'none':
        pytest.fail('@internet_off test ran when remote_data!=none')



# Licensed under a 3-clause BSD style license - see LICENSE.rst

import os
import pytest

PYFILE_CONTENTS = """
    import pytest
    from six.moves.urllib.request import urlopen

    def test_config_setting(pytestconfig):
        assert pytestconfig.getini('remote_data_strict') == {}

    {}
    def test_internet_access():
        urlopen('http://astropy.org')
    """

def _write_config_file(testdir, entry):
    config = testdir.tmpdir.join('setup.cfg')
    config.write("""
[pytest]
{}
        """.format(entry))

    # Just a sanity check to make sure we actually write the config file
    assert os.path.exists(str(testdir.tmpdir.join('setup.cfg')))


def test_local_config(pytestconfig):
    assert pytestconfig.getini('remote_data_strict') == True


def test_default_behavior(testdir):
    _write_config_file(testdir, '')

    testdir.makepyfile(PYFILE_CONTENTS.format('False', ''))

    result = testdir.runpytest_subprocess()
    result.assert_outcomes(passed=2)


def test_strict_behavior(testdir):
    _write_config_file(testdir, 'remote_data_strict = true')

    testdir.makepyfile(PYFILE_CONTENTS.format('True', ''))

    result = testdir.runpytest_subprocess()
    result.assert_outcomes(passed=1, failed=1)


@pytest.mark.parametrize('source', ['none', 'any'])
def test_strict_with_decorator(testdir, source):
    _write_config_file(testdir, 'remote_data_strict = true')

    decorator = '@pytest.mark.remote_data'
    testdir.makepyfile(PYFILE_CONTENTS.format('True', decorator))

    clarg = '--remote-data=' + source
    result = testdir.runpytest_subprocess(clarg)

    if source == 'none':
        outcomes = dict(passed=1, skipped=1)
    else:
        outcomes = dict(passed=2)
    result.assert_outcomes(**outcomes)
